"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const fs_1 = require("./fs");
class FileDatabase {
    constructor(pathOrOptions) {
        this.leftPad = 0;
        this.prefix = '$';
        this.jsonSpaces = 0;
        let options;
        if (typeof pathOrOptions === 'string') {
            options = { path: pathOrOptions };
        }
        else {
            options = pathOrOptions;
        }
        if (!options.path) {
            throw new Error('Missing path!');
        }
        this.setOptions(options);
    }
    setOptions(options) {
        this.path = options.path || this.path;
        this.leftPad = options.leftPad || this.leftPad;
    }
    async get(path, options) {
        const depth = (options && options.depth) || 0;
        const resolvedPath = this.resolvePath(path);
        const [file, dir] = await Promise.all([
            this.readFile(resolvedPath + '.json'),
            this.readDir(resolvedPath),
        ]);
        let result = {
            [`${this.prefix}path`]: path,
        };
        if (file) {
            Object.assign(result, file);
        }
        if (dir.content) {
            if (dir.metadata.type === 'array') {
                result = await Promise.all(dir.content.map(c => this.get(path_1.join(path, c), { depth })));
            }
            else if (depth === 0) {
                for (const c of dir.content) {
                    Object.assign(result, { [c]: null });
                }
            }
            else {
                const content = await Promise.all(dir.content.map(c => this.get(path_1.join(path, c), { depth: depth - 1 })));
                for (let i = 0; i < content.length; ++i) {
                    Object.assign(result, {
                        [dir.content[i]]: content[i],
                    });
                }
            }
        }
        return result;
    }
    async set(path, object) {
        path = this.resolvePath(path);
        this.writeFile(path + '.json', object);
    }
    async readDir(directoryPath) {
        let dir = [];
        try {
            dir = await fs_1.readdir(directoryPath);
        }
        catch (err) { }
        let metadata = { type: 'object' };
        const content = new Set();
        for (const entry of dir) {
            if (entry === '_metadata.json') {
                metadata = await this.readFile(path_1.join(directoryPath, entry));
            }
            else if (entry.endsWith('.json')) {
                content.add(entry.substr(0, entry.length - '.json'.length));
            }
            else {
                content.add(entry);
            }
        }
        return { metadata, content: Array.from(content) };
    }
    async readFile(filename) {
        try {
            const file = await fs_1.readFile(filename);
            return JSON.parse(file.toString());
        }
        catch (err) { }
    }
    async writeFile(filename, content) {
        try {
            const serialized = JSON.stringify(content, undefined, this.jsonSpaces);
            await fs_1.writeFile(filename, serialized);
        }
        catch (err) {
            console.warn(err);
        }
    }
    resolvePath(path) {
        if (this.leftPad) {
            let filename = path_1.basename(path);
            if (Number(filename).toString() === filename) {
                filename = filename.padStart(this.leftPad, '0');
            }
            path = path_1.join(path, '../', filename);
        }
        return path_1.resolve(this.path, path);
    }
}
exports.FileDatabase = FileDatabase;
//# sourceMappingURL=FileDatabase.js.map